/* eslint-env jest */
import { SldsButton } from '../button';
import * as utils from '../../../../../.jest/utils';
// Register custom element for jest wrapper
customElements.define('slds-button', SldsButton);

describe('slds-button', () => {
  afterEach(() => {
    // The jsdom instance is shared across test cases in a single file so reset the DOM
    while (document.body.firstChild) {
      document.body.removeChild(document.body.firstChild);
    }
  });
  it('named part exists on button element', async () => {
    const element = utils.renderIntoBody(`<slds-button>Test</slds-button>`);
    const button = element.shadowRoot.querySelector('button');
    expect(button).toHaveAttribute('part', 'button');
  });
  it('named part exists when start slot is used', async () => {
    const element = utils.renderIntoBody(`<slds-button><span slot="start">Start</span>Test</slds-button>`);
    const startSlot = element.shadowRoot.querySelector('[part~="start"]');
    expect(startSlot).toHaveAttribute('part', 'start');
  });
  it('named part exists when end slot is used', async () => {
    const element = utils.renderIntoBody(`<slds-button><span slot="end">End</span>Test</slds-button>`);
    const endSlot = element.shadowRoot.querySelector('[part~="end"]');
    expect(endSlot).toHaveAttribute('part', 'end');
  });
  it('named parts do not exist when slots are not used', async () => {
    const element = utils.renderIntoBody(`<slds-button>Test</slds-button>`);
    const startSlot = element.shadowRoot.querySelector('[part~="start"]');
    const endSlot = element.shadowRoot.querySelector('[part~="end"]');
    expect(startSlot).not.toBeInTheDocument();
    expect(endSlot).not.toBeInTheDocument();
  });
  it('size attribute exists on slds element', async () => {
    const element = utils.renderIntoBody(`<slds-button size="stretch">Test</slds-button>`);
    expect(element).toHaveAttribute('size', 'stretch');
  });
  it('variant attribute exists on slds element', async () => {
    const element = utils.renderIntoBody(`<slds-button variant="brand">Test</slds-button>`);
    expect(element).toHaveAttribute('variant', 'brand');
  });
  it('disabled attribute delegates to button element', async () => {
    const element = utils.renderIntoBody(`<slds-button disabled>Test</slds-button>`);
    const button = element.shadowRoot.querySelector('button');
    expect(element).not.toHaveAttribute('disabled', '');
    expect(button).toHaveAttribute('disabled', '');
  });
  it('default attributes exists on slds element', async () => {
    const element = utils.renderIntoBody(`<slds-button>Test</slds-button>`);
    const button = element.shadowRoot.querySelector('button');
    expect(button).not.toHaveAttribute('disabled', '');
    expect(element).toHaveAttribute('variant', 'neutral');
  });
  it('stretch attribute exists on slds element', async () => {
    const element = utils.renderIntoBody(`<slds-button stretch>Test</slds-button>`);
    expect(element).toHaveAttribute('stretch');
  });
});
