import { fragmentFrom } from 'elix/src/core/htmlLiterals.js';
import { defaultState, firstRender, render, template, setState, state } from 'elix/src/base/internal';
import SdsButton from '@salesforce-ux/sds-button/button';
import css from './button.module.css';

class SldsButton extends SdsButton {
  get [defaultState]() {
    return Object.assign(super[defaultState], {
      size: null,
      variant: 'neutral',
    });
  }

  get size() {
    return this[state].size;
  }

  set size(size) {
    this[setState]({
      size: size,
    });
  }

  [render](changed) {
    super[render](changed);

    /**
     * Imitate conditional slot rendering akin to LBC
     */
    if (firstRender) {
      const slotStart = this.shadowRoot.querySelector('slot[name="start"]');
      const slotEnd = this.shadowRoot.querySelector('slot[name="end"]');
      if (slotStart && !slotStart.assignedElements()[0]) slotStart.remove();
      if (slotEnd && !slotEnd.assignedElements()[0]) slotEnd.remove();
    }

    if (changed.size) {
      const { size } = this[state];
      this.reflectAttribute(this, 'size', size);
    }

    if (changed.variant) {
      const { variant } = this[state];
      this.reflectAttribute(this, 'variant', variant);
    }
  }

  get [template]() {
    const result = super[template];
    result.content.append(fragmentFrom.html`
      <style>
        ${css}
      </style>
    `);

    return result;
  }

  reflectAttribute(element, name, value) {
    if (value) {
      element.setAttribute(name, value);
    } else {
      element.removeAttribute(name);
    }
  }
}

export { SldsButton };
